﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Threading;
using System.Windows.Media.Animation;

namespace MyWpfApp
{
    public class MyPanel:Panel
    {
        Random rand = null;
        DispatcherTimer Timer = null;

        public MyPanel()
        {
            rand = new Random();
            Timer = new DispatcherTimer();
            Timer.Interval = TimeSpan.FromSeconds(SwapInterval);
            Timer.Tick += Timer_Tick;
            Timer.Start();
        }

        #region 属性
        public static readonly DependencyProperty SwapIntervalProperty = DependencyProperty.Register("SwapInterval", typeof(double), typeof(MyPanel), new PropertyMetadata(10d, new PropertyChangedCallback(SwapIntervalChanged), new CoerceValueCallback(CoerceValCallback)));

        private static void SwapIntervalChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            MyPanel p = d as MyPanel;
            double sec = (double)e.NewValue;
            p.Timer.Stop();
            p.Timer.Interval = TimeSpan.FromSeconds(sec);
            p.Timer.Start();
        }

        private static object CoerceValCallback(DependencyObject d, object baseValue)
        {
            // 通过该方法测检依赖项属性的值
            // 如果设置的值小于5秒，就自动改为5秒
            double dv = (double)baseValue;
            if (dv < 5d)
            {
                dv = 5d;
            }
            return dv;
        }

        /// <summary>
        /// 切换动画间隔，以秒为单位
        /// </summary>
        public double SwapInterval
        {
            get { return (double)GetValue(SwapIntervalProperty); }
            set { SetValue(SwapIntervalProperty, value); }
        }
        #endregion

        void Timer_Tick(object sender, EventArgs e)
        {
            DoubleAnimation dat = new DoubleAnimation();
            dat.From = 1d;
            dat.To = 0d;
            dat.Duration = TimeSpan.FromMilliseconds(900);
            dat.Completed += (sd, arg) =>
                {
                    // 当动画完成时，面板的Opacity为0
                    // 此时重新排列子元素
                    this.InvalidateArrange();
                    DoubleAnimation datBack = new DoubleAnimation();
                    datBack.From = 0d;
                    datBack.To = 1d;
                    datBack.Duration = TimeSpan.FromSeconds(1);
                    // 子元素排列完成后，再执行一个动画
                    // 将Opacity再改为1
                    this.BeginAnimation(OpacityProperty, datBack);
                };
            // 开始动画，隐藏面板
            this.BeginAnimation(OpacityProperty, dat);
        }

        protected override Size MeasureOverride(Size availableSize)
        {
            // 处理子元素的大小
            foreach (UIElement u in InternalChildren)
            {
                // 一定要对每个子元素调用Measure
                // 不然，就看不到子元素了
                u.Measure(availableSize);
            }
            // 如果不要求精确计算子元素占了多少
            // 空间，可以直接返回0-0的Size，但不
            // 要返回正无穷大，否则后果自负
            return new Size();
        }

        protected override Size ArrangeOverride(Size finalSize)
        {
            // 通过该方法对子元素进行排列
            foreach (UIElement item in InternalChildren)
            {
                // 算出子元素的坐标的随机值
                double maxX = finalSize.Width - item.DesiredSize.Width;
                double maxY = finalSize.Height - item.DesiredSize.Height;
                if (maxX <=0d)
                {
                    maxX = 1d;
                }
                if (maxY <= 0d)
                {
                    maxY = 1d;
                }
                double X = rand.Next(0, (int)maxX);
                double Y = rand.Next(0, (int)maxY);
                // 调用Arrange方法安排子元素放在哪个位置
                item.Arrange(new Rect(X, Y, item.DesiredSize.Width, item.DesiredSize.Height));
            }
            return finalSize;
        }
    }
}
